#include "tinyexpr.h"
#include <stdio.h>
#include <ctype.h>

void tolowercase(const char *s) {
    while (*s != '\0') {
        *s = tolower(*s);
        ++s;
    }
}

int main(int argc, char *argv[])
{
    if (argc < 2) {
        printf("Usage: example2 \"expression\"\n");
        return 0;
    }

    const char *expression = argv[1];
    tolowercase(expression);
    printf("Evaluating:\n\t%s\n", expression);

    /* This shows an example where the variables
     * x and y are bound at eval-time. */
    double x, y;
/*
typedef struct te_variable {
    const char *name;
    const void *address;
    int type;
    void *context;
} te_variable;
*/
    te_variable vars[2];
    vars[0].name = "x";
    vars[0].address = &x;
    vars[1].name = "y";
    vars[1].address = &y;
//    = {{"x", &x}, {"y", &y}};

    /* This will compile the expression and check for errors. */
    int err;
    te_expr *n = te_compile(expression, vars, 2, &err);

    if (n) {
        /* The variables can be changed here, and eval can be called as many
         * times as you like. This is fairly efficient because the parsing has
         * already been done. */
        x = 3; y = 4;
        const double r = te_eval(n); printf("Result:\n\t%f\n", r);

        te_free(n);
    } else {
        /* Show the user where the error is at. */
        printf("\t%*s^\nError near here", err-1, "");
    }


    return 0;
}
